/*
** a_alloc.c - Allocation routines that either succeeds or abort.
**
** Copyright (c) 1997-2002 Peter Eriksson <pen@lysator.liu.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#ifndef PLIB_AALLOC_H

#include <stdio.h>
#include <sys/types.h>

#ifdef DEBUG
#ifndef PLIB_IN_ALLOC_C
#define malloc #error
#define realloc #error
#define calloc #error
#define free #error
#define strdup #error
#define strndup #error
#endif
#endif

#define A_ALLOC(VAR)   a_malloc((VAR), #VAR)
#define A_NEW(VAR)     ((VAR) = a_malloc(sizeof(*(VAR)), #VAR))
#define A_STRDUP(VAR)  a_strdup((VAR), #VAR)


extern void
a_init(void);

extern void
a_dump(FILE *fp);

extern void *
a_malloc(size_t size,
	 const char *what);

extern void *
a_realloc(void *oldp,
	  size_t nsize,
	  const char *what);

extern void
a_free(void *p);

extern char *
a_strndup(const char *s,
	  size_t len,
	  const char *what);

extern char *
a_strdup(const char *s,
	 const char *what);


#endif


